#####################################################################################################
# Librerías
#####################################################################################################
# Data Handle
import pandas as pd
import numpy as np
import random

# Viz
import matplotlib.pyplot as plt
import seaborn as sns
import plotly.express as px
import squarify

# Dir
import os
import sys

# Format
pd.set_option('display.float_format','{:.2f}'.format)

# Seed
seed = 42

# Configuración de la semilla aleatoria para reproducibilidad
np.random.seed(42)

#####################################################################################################
# EDA
#####################################################################################################
def eda_summary(df):
    # Imprimir las dimensiones del dataset
    print("Dimensiones del dataset:")
    print(f"Filas: {df.shape[0]}, Columnas: {df.shape[1]}\n")

    # Imprimir la lista de las variables
    print("Variables en el dataset:")
    print(df.columns.tolist(), "\n")

    # Dtypes de las variables
    print("Tipos de datos de las variables:")
    print(df.dtypes, "\n")

    # Nulos por variable
    missing_data = df.isnull().sum()
    missing_percent = (missing_data / len(df))
    missing_table = pd.DataFrame({
        'Cantidad de Nulos': missing_data,
        '% de Nulos': missing_percent
    })
    print("Nulos por variable:")
    display(missing_table)

    # Estadísticas de las variables numéricas
    print("Estadísticas de las variables numéricas:")
    display(df.describe(include=[np.number]).T.round(1))

    # Estadísticas de las variables categóricas
    print("Estadísticas de las variables categóricas:")
    display(df.describe(include=['object', 'category', 'bool']).T)

#####################################################################################################
# Plot Numeric
#####################################################################################################
def plot_numeric_variables(df, target_var):
    # Filtrar por columnas numéricas
    numeric_columns = df.select_dtypes(include=[np.number]).columns.tolist()
    
    # Configurar el estilo de seaborn
    sns.set(style='whitegrid')

    # Matriz de correlaciones
    corr_matrix = df[numeric_columns].corr()
    
    # Máscara para la parte superior de la matriz (triangular superior)
    mask = np.triu(np.ones_like(corr_matrix, dtype=bool))

    # Plot de la matriz de correlaciones
    plt.figure(figsize=(10, 8))
    sns.heatmap(corr_matrix, mask=mask, annot=True, fmt=".2f", cmap='coolwarm', cbar=True)
    plt.title('Matriz de Correlaciones Triangular Inferior')
    plt.show()

    # Generar los gráficos para cada variable numérica
    for column in numeric_columns:
        if column != target_var:  # Asegurarse de no graficar la variable objetivo contra sí misma
            fig, axes = plt.subplots(1, 2, figsize=(18, 5))

            # Plot 1: Histograma y curva de densidad
            sns.histplot(df[column], kde=True, ax=axes[0])
            axes[0].set_title(f'Distribución de {column}')
            axes[0].set_xlabel(column)
            axes[0].set_ylabel('Frecuencia')

            # Plot 2: Boxplot
            sns.boxplot(x=df[column], ax=axes[1])
            axes[1].set_title(f'Boxplot de {column}')
            axes[1].set_xlabel(column)
            axes[1].set_ylabel('Valor')

            plt.tight_layout()
            plt.show()

#####################################################################################################
# Plot Categorical
#####################################################################################################
def plot_categorical_variables(df, target_var):
    # Filtrar por columnas categóricas
    categorical_columns = df.select_dtypes(include=['object', 'category', 'bool']).columns.tolist()
    
    # Configurar el estilo de seaborn
    sns.set(style='whitegrid')
    
    # Generar los gráficos para cada variable categórica
    for column in categorical_columns:
        # Datos para el treemap y el gráfico de barras
        category_counts = df[column].value_counts()

        # Configurar el layout de los subplots
        fig, axes = plt.subplots(1, 2, figsize=(24, 6))

        # Plot 1: Gráfico de Barras en subplot
        sns.barplot(x=category_counts.index, y=category_counts.values, palette="Set3", ax=axes[0])
        axes[0].set_title(f'Distribución de {column}')
        axes[0].set_xlabel(column)
        axes[0].set_ylabel('Frecuencia')
        axes[0].tick_params(axis='x', rotation=45)

        # Plot 2: Boxplot de la variable objetivo por cada categoría en subplot
        sns.boxplot(x=column, y=target_var, data=df, palette="Set3", ax=axes[1])
        axes[1].set_title(f'Boxplot de {target_var} por {column}')
        axes[1].set_xlabel(column)
        axes[1].set_ylabel(target_var)
        axes[1].tick_params(axis='x', rotation=45)

        # Ajustar el layout para evitar la superposición de etiquetas
        fig.tight_layout()
        plt.show()